"""
Utils module
"""

import shutil
from pathlib import Path
from typing import Any, List, Tuple

import numpy as np
from face_recognition import (face_distance, face_encodings, face_locations,
                              load_image_file)
from face_recognition.face_recognition_cli import image_files_in_folder
from rich.progress import track

IR_DIR = "ir"
RGB_DIR = "rgb"


def _copy_images_by_dict(
    faces_paths: List[List[Path]],
    input_dir: Path,
    output_dir: Path,
):
    for num, img_paths in track(
        enumerate(faces_paths), "Saving...", total=len(faces_paths)
    ):
        path_rgb = output_dir / RGB_DIR / str(num)
        path_ir = output_dir / IR_DIR / str(num)

        if not path_rgb.exists():
            path_rgb.mkdir(parents=True)
        if not path_ir.exists():
            path_ir.mkdir(parents=True)

        for img_path in img_paths:
            rgb_name = img_path.name
            time = _get_timestamp(rgb_name)
            ir_name = f"infra_{time}.jpg"
            dst_rgb = path_rgb / rgb_name
            dst_ir = path_ir / ir_name
            shutil.copyfile(img_path, dst_rgb)
            shutil.copyfile(input_dir / IR_DIR / ir_name, dst_ir)


def _get_timestamp(name: str) -> str:
    return name.split("_")[-1].split(".")[0]


def _sort_path_by_encodings(
    np_face: np.ndarray, np_path: np.ndarray
) -> List[List[Path]]:
    """Сортирует пути по похожим энкодингам в словарь"""

    faces: List[List[Path]] = []
    while len(np_face) > 0:
        current, np_face = _np_pop(np_face)
        path, np_path = _np_pop(np_path)
        indexes = face_distance(np_face, current) <= 0.6
        faces.append(list(np_path[indexes]) + [path])
        np_face = np_face[~indexes]
        np_path = np_path[~indexes]
    return faces


def _np_pop(array: np.ndarray) -> Tuple[Any, np.ndarray]:
    """Операция pop для ndarray"""

    head = array[-1]
    tail = array[:-1]
    return head, tail


def _get_encodings_from_image_dir(
    input_dir: Path, verbose: bool
) -> Tuple[np.ndarray, np.ndarray]:
    """Извлекает энкодинги и возвращает вместе с путями"""
    face_encs: List[np.ndarray] = []
    face_path = []
    for img_path in track(image_files_in_folder(input_dir / RGB_DIR), "Processing..."):
        image = load_image_file(img_path)
        face_bounding_boxes = face_locations(image)

        if len(face_bounding_boxes) != 1:
            # If there are no people (or too many people) in a training image,
            # skip the image.
            if verbose:
                print(
                    "Image {} not suitable for training: {}".format(
                        img_path,
                        "Didn't find a face"
                        if len(face_bounding_boxes) < 1
                        else "Found more than one face",
                    )
                )
        else:
            # Add face encoding for current image to list
            face_encs.append(
                face_encodings(image, known_face_locations=face_bounding_boxes)[0]
            )
            face_path.append(Path(img_path))

    return np.array(face_encs), np.array(face_path)



"""
Command line module
"""
import time

import click
from click import Path

from face_group.api import process_images


@click.command()
@click.option(
    "-i",
    "--input-dir",
    "input_dir",
    type=Path(exists=True),
    help="Path of directory with rgb and infra images.",
)
@click.option(
    "-o",
    "--output-dir",
    "output_dir",
    default="out",
    type=Path(),
    help="Path of output directory (default='out').",
)
@click.option(
    "-v", "--verbose", "verbose", default=True, type=bool, help="Show info or not."
)
def group_faces(input_dir: str, output_dir: str, verbose: bool) -> None:
    """It groups the faces of similar people"""

    start = time.time()
    process_images(input_dir, output_dir, verbose)
    click.echo(f"Done in {time.time() - start}s")



"""
API module
"""

from pathlib import Path, PurePath
from typing import Union

from face_group.utils import (_copy_images_by_dict,
                              _get_encodings_from_image_dir,
                              _sort_path_by_encodings)


def process_images(
    input_dir: Union[str, PurePath],
    output_dir: Union[str, PurePath],
    verbose: bool = False,
) -> None:
    """Про"""
    input_dir = Path(input_dir)
    output_dir = Path(output_dir)

    if not output_dir.exists():
        output_dir.mkdir()

    np_face, np_path = _get_encodings_from_image_dir(input_dir, verbose)

    face_dict = _sort_path_by_encodings(np_face, np_path)

    _copy_images_by_dict(face_dict, input_dir, output_dir)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['face_group']

package_data = \
{'': ['*']}

install_requires = \
['face-recognition>=1.3.0,<2.0.0', 'rich>=9.11.1,<10.0.0']

entry_points = \
{'console_scripts': ['face-group = face_group.cli:group_faces']}

setup_kwargs = {
    'name': 'face-group',
    'version': '0.1.3',
    'description': 'Simple project for grouping faces of people by encodings.',
    'long_description': '# Face group\n> Grouping rgb and infra people`s faces by folders using face features\n\n`face-group` finds faces using dlib\n\nParameters:\n- *input-dir* - path of directory with rgb and infra images\n- *output-dir* - path of output directory (default="out")\n- *verbose* - show info or not\n\n## Installation\n```\npip install face-group\n```\n## Usage\n```\n$ face-group --help\nUsage: face-group [OPTIONS]\n\n  It groups the faces of similar people\n\nOptions:\n  -i, --input-dir PATH   Path of directory with rgb and infra images.\n  -o, --output-dir PATH  Path of output directory (default=\'out\').\n  -v, --verbose BOOLEAN  Show info or not.\n  --help                 Show this message and exit.\n```\n',
    'author': 'irlirion',
    'author_email': 'irlirion@protonmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)
